#include "model.h"

#include <QMimeData>
#include <QBrush>

/* ---------------------------------------------------------------------- */

TreeModel::TreeModel (QGraphicsItem * root, QObject * parent)
   : QAbstractItemModel (parent),
     rootItem (root)
{
}

TreeModel::~TreeModel ()
{
}

/* ---------------------------------------------------------------------- */

int TreeModel::columnCount (const QModelIndex & parent) const
{
   return 1;
}

QVariant TreeModel::data (const QModelIndex & index, int role) const
{
   if (! index.isValid ())
      return QVariant ();

   QGraphicsItem * item = static_cast <QGraphicsItem * > (index.internalPointer ());

   return item->data (role);
}

#if 0
QVariant TreeModel::headerData (int section, Qt::Orientation orientation, int role) const
{
   if (orientation == Qt::Horizontal && role == Qt::DisplayRole)
      return "title";

   return QVariant ();
}
#endif

/* ---------------------------------------------------------------------- */

QModelIndex TreeModel::index (int row, int column, const QModelIndex & parent) const
{
   if (! hasIndex (row, column, parent))
      return QModelIndex ();

   QGraphicsItem * parentItem;

   if (! parent.isValid ())
      parentItem = rootItem;
   else
      parentItem = static_cast <QGraphicsItem*> (parent.internalPointer ());

   QGraphicsItem * childItem = NULL;
   if (parentItem != NULL)
   {
      QList<QGraphicsItem *> items = parentItem->childItems ();
      if (row < items.length())
         childItem = items [row];
   }

   if (childItem != NULL)
      return createIndex (row, column, childItem);
   else
      return QModelIndex ();
}

QModelIndex TreeModel::parent (const QModelIndex & index) const
{
   if (!index.isValid ())
      return QModelIndex ();

   QGraphicsItem * childItem = static_cast <QGraphicsItem*> (index.internalPointer ());
   QGraphicsItem * parentItem = childItem->parentItem ();

   if (parentItem == NULL || parentItem == rootItem)
      return QModelIndex ();

   QList<QGraphicsItem *> items = parentItem->childItems ();
   int cnt = items.length ();

   int pos = 0;
   while (pos < cnt && items [pos] != childItem)
   {
      pos ++;
   }

   return createIndex (pos, 0, parentItem);
}

int TreeModel::rowCount (const QModelIndex & parent) const
{
   QGraphicsItem * parentItem;
   if (parent.column () > 0)
      return 0;

   if (! parent.isValid ())
      parentItem = rootItem;
   else
      parentItem = static_cast <QGraphicsItem*> (parent.internalPointer ());

   int cnt = 0;

   if (parentItem != NULL)
   {
      cnt = parentItem->childItems().length();
   }

   return cnt;
}

/* ---------------------------------------------------------------------- */

Qt::ItemFlags TreeModel::flags (const QModelIndex & index) const
{
   if (! index.isValid ())
      return Qt::ItemIsDropEnabled;

   return Qt::ItemIsEnabled | Qt::ItemIsSelectable | Qt::ItemIsDragEnabled | Qt::ItemIsDropEnabled ;
}

Qt::DropActions TreeModel::supportedDropActions () const
{
     return Qt::CopyAction | Qt::MoveAction | Qt::LinkAction;
}

QStringList TreeModel::mimeTypes() const
{
   QStringList types;
   types << "application/x-color";
   return types;
}

bool TreeModel::dropMimeData (const QMimeData * data, Qt::DropAction action,
                              int row, int column, const QModelIndex & parent)
{
   bool ok = false;
   if (data->hasColor ())
   {
      QColor color = data->colorData ().value <QColor> ();

      if (parent.isValid ())
      {
          QGraphicsItem * item = static_cast < QGraphicsItem * > (parent.internalPointer ());
          QAbstractGraphicsShapeItem * shape = dynamic_cast < QAbstractGraphicsShapeItem * > (item);
          if (shape != NULL)
          {
             shape->setBrush (QBrush (color));
             ok = true;
          }
      }
   }
   return ok;
}

/* ---------------------------------------------------------------------- */

