#ifndef WINDOW_H
#define WINDOW_H

#include <QMainWindow>

#include <QToolButton>
#include <QTableWidgetItem>

#include <QXmlStreamWriter>

#include <QGraphicsScene>
#include <QAbstractGraphicsShapeItem>
#include <QGraphicsRectItem>
#include <QGraphicsEllipseItem>

#include "model.h"

namespace Ui {
class MainWindow;
}

/* ---------------------------------------------------------------------- */

class ParamList : public QMap <QString, QString>
{
   public:
      int number (QString key, int default_value = 0);
};

/* ---------------------------------------------------------------------- */

class Common : public QObject
{
Q_OBJECT


};

class Rectangle : public Common, public QGraphicsRectItem
{
Q_OBJECT
public:
   // Q_PROPERTY (QString name READ objectName WRITE setObjectName)
   Q_PROPERTY (int x READ x WRITE setX)
   Q_PROPERTY (int y READ y WRITE setY)
   Q_PROPERTY (int width READ width WRITE setWidth)
   Q_PROPERTY (int height READ height WRITE setHeight)
   Q_PROPERTY (QColor pen READ penColor WRITE setPen)
   Q_PROPERTY (QColor brush READ brushColor WRITE setBrush)

   int width () { return rect().width(); }
   void setWidth (int w) { auto r = rect (); r.setWidth (w); setRect (r); }

   int height () { return rect().height(); }
   void setHeight (int h) { auto r = rect (); r.setHeight(h); setRect (r); }

   QColor penColor () { return pen().color(); }
   QColor brushColor () { return brush().color(); }

protected:
    void dragEnterEvent (QGraphicsSceneDragDropEvent *event);
    void dropEvent (QGraphicsSceneDragDropEvent *event);
public:
    Rectangle ();
    ParamList params;
};

class Ellipse : public Common, public QGraphicsEllipseItem
{
Q_OBJECT
public:
   Q_PROPERTY (int x READ x WRITE setX)
   Q_PROPERTY (int y READ y WRITE setY)
   Q_PROPERTY (int width READ width WRITE setWidth)
   Q_PROPERTY (int height READ height WRITE setHeight)
   Q_PROPERTY (QColor pen READ penColor WRITE setPen)
   Q_PROPERTY (QColor brush READ brushColor WRITE setBrush)

   int width () { return rect().width(); }
   void setWidth (int w) { auto r = rect (); r.setWidth (w); setRect (r); }

   int height () { return rect().height(); }
   void setHeight (int h) { auto r = rect (); r.setHeight(h); setRect (r); }

   QColor penColor () { return pen().color(); }
   QColor brushColor () { return brush().color(); }

protected:
    void dragEnterEvent (QGraphicsSceneDragDropEvent *event);
    void dropEvent (QGraphicsSceneDragDropEvent *event);
public:
    Ellipse ();
    ParamList params;
};

/* ---------------------------------------------------------------------- */

class ColorButton : public QToolButton
{
private :
    QString name;
    QColor color;
public :
    ColorButton (QString s);
protected :
    void mousePressEvent(QMouseEvent *event);
private :
    QPixmap createPixmap (bool white = false);
};

/* ---------------------------------------------------------------------- */

class ShapeButton : public QToolButton
{
private :
    QString name;
    QString code;
public :
    ShapeButton (QString p_name, QString p_code);
protected :
    void mousePressEvent(QMouseEvent *event);
};

/* ---------------------------------------------------------------------- */

class MainWindow : public QMainWindow
{
    Q_OBJECT

public:
    explicit MainWindow(QWidget *parent = 0);
    ~MainWindow();

public:
   void addGraphicsItem (QGraphicsItem * target, QString code, bool plus);

private:
    void readFile (QString fileName);
    void writeFile (QString fileName);
    void writeItem (QXmlStreamWriter &writer, QGraphicsItem * item);

    void addIntoScene (QGraphicsItem * target, QGraphicsItem * item);
    void readXml (QXmlStreamReader & reader, QGraphicsItem * target);

    void showTable (QGraphicsItem * item);

private slots:
    // void on_treeWidget_itemClicked (QTreeWidgetItem *item, int column);

    void on_actionOpen_triggered();

    void on_actionSave_triggered();

    void on_actionQuit_triggered();

    void on_tableWidget_itemChanged (QTableWidgetItem *item);

    void on_treeView_clicked(const QModelIndex &index);

private:
    Ui::MainWindow * ui;

    QGraphicsScene * scene;
    QGraphicsItem * top_item;
    QGraphicsItem * current_item;
};

/* ---------------------------------------------------------------------- */

#endif // MAINWINDOW_H
